from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

from polars import Expr
from polars.plugins import register_plugin_function

from witt_strutil._internal import __version__ as __version__

if TYPE_CHECKING:
    from witt_strutil.typing import IntoExprColumn

LIB = Path(__file__).parent

def pig_latinnify(expression: IntoExprColumn) -> Expr:
    return register_plugin_function(plugin_path    = LIB,
                                    function_name  = 'pig_latinnify',
                                    args           = [expression],
                                    is_elementwise = True)

def split_by_chars(expression: IntoExprColumn, characters: str) -> Expr:
    return register_plugin_function(plugin_path    = LIB,
                                    function_name  = 'split_by_chars',
                                    args           = [expression],
                                    kwargs         = {'characters': characters},
                                    is_elementwise = True)

def split_by_lowercase_to_uppercase(expression: IntoExprColumn) -> Expr:
    return register_plugin_function(plugin_path    = LIB,
                                    function_name  = 'split_by_lowercase_to_uppercase',
                                    args           = [expression],
                                    is_elementwise = True)

def split_by_uppercase_to_lowercase(expression: IntoExprColumn) -> Expr:
    return register_plugin_function(plugin_path    = LIB,
                                    function_name  = 'split_by_uppercase_to_lowercase',
                                    args           = [expression],
                                    is_elementwise = True)

def split_by_digit_to_nondigit(expression: IntoExprColumn) -> Expr:
    return register_plugin_function(plugin_path    = LIB,
                                    function_name  = 'split_by_digit_to_nondigit',
                                    args           = [expression],
                                    is_elementwise = True)

def split_by_nondigit_to_digit(expression: IntoExprColumn) -> Expr:
    return register_plugin_function(plugin_path    = LIB,
                                    function_name  = 'split_by_nondigit_to_digit',
                                    args           = [expression],
                                    is_elementwise = True)

def to_sentence_case(expression: IntoExprColumn) -> Expr:
    return register_plugin_function(plugin_path    = LIB,
                                    function_name  = 'to_sentence_case',
                                    args           = [expression],
                                    is_elementwise = True)

def to_sponge_case(expression: IntoExprColumn) -> Expr:
    return register_plugin_function(plugin_path    = LIB,
                                    function_name  = 'to_sponge_case',
                                    args           = [expression],
                                    is_elementwise = True)


# test_to_sponge_case.py
#
# Copyright (c) 2025 Naufan Rusyda Faikar
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: AGPL-3.0-or-later

from witt_strutil import to_sponge_case
import polars

def test_to_sponge_case():
    df = polars.DataFrame({
        'input': ['lorem. ipsum! dolor? sit amet.'],
    })
    df = df.with_columns(output=to_sponge_case('input'))

    assert True # no idea how to test this :/


# test_to_sentence_case.py
#
# Copyright (c) 2025 Naufan Rusyda Faikar
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: AGPL-3.0-or-later

from witt_strutil import to_sentence_case
import polars

def test_to_sentence_case():
    df = polars.DataFrame({
        'input': [
            'lorem. ipsum! dolor? sit amet.',
            'lorem.ipsum!dolor?sit amet.',
            'UPPERCASE',
            'lowercase',
            'kebab-case',
            'snake_case',
            'camelCase',
            'PascalCase',
            'CONSTANT_CASE',
            'dot.case',
            'Sentence case',
            None,
        ],
        'expected': [
            'Lorem. Ipsum! Dolor? Sit amet.',
            'Lorem.ipsum!dolor?sit amet.',
            'Uppercase',
            'Lowercase',
            'Kebab-case',
            'Snake_case',
            'Camel case',
            'Pascal case',
            'Constant_case',
            'Dot.case',
            'Sentence case',
            None,
        ],
    })
    df = df.with_columns(output=to_sentence_case('input'))

    assert df['output'].to_list() == df['expected'].to_list()


# test_split_by_uppercase_to_lowercase.py
#
# Copyright (c) 2025 Naufan Rusyda Faikar
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: AGPL-3.0-or-later

from witt_strutil import split_by_uppercase_to_lowercase
import polars

def test_split_by_uppercase_to_lowercase():
    df = polars.DataFrame({
        'input': [
            'SymbianOS',
            'WinCE',
            'webOS',
            'iPhone',
            'JSONfile',
            'GObject',
            'Python',
            'Sony Ericsson',
            'BlackBerry OS',
        ],
        'expected': [
            ['S', 'ymbianOS'],
            ['W', 'inCE'],
            ['webOS'],
            ['iP', 'hone'],
            ['JSON', 'file'],
            ['GO', 'bject'],
            ['P', 'ython'],
            ['S', 'ony E', 'ricsson'],
            ['B', 'lackB', 'erry OS'],
        ],
    })
    df = df.with_columns(output=split_by_uppercase_to_lowercase('input'))

    assert df['output'].to_list() == df['expected'].to_list()


# test_pig_latinnify.py
#
# Copyright (c) 2025 Naufan Rusyda Faikar
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: AGPL-3.0-or-later

from witt_strutil import pig_latinnify
import polars

def test_pig_latinnify():
    df = polars.DataFrame({
        'input': [
            'he does not know',
            'this',
            'is',
            'banana',
            'black',
            'smile',
            'straight',
            'hello!',
        ],
        'expected': [
            'ehay oesday otnay owknay',
            'isthay',
            'isway',
            'ananabay',
            'ackblay',
            'ilesmay',
            'aightstray',
            'ellohay!',
        ],
    })
    df = df.with_columns(output=pig_latinnify('input'))

    assert df['output'].to_list() == df['expected'].to_list()


# test_split_by_chars.py
#
# Copyright (c) 2025 Naufan Rusyda Faikar
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: AGPL-3.0-or-later

from witt_strutil import split_by_chars
import polars

def test_split_by_chars():
    input = polars.DataFrame({
        'input': [
            'Nadya Arina, Refal Hady, Giorgino Abraham, Anggika Bolsterli, Laura Theux, Christine Hakim',
            'Maudy Koesnaedi, Rano Karno, Cornelia Agatha, Mandra Naih, Aminah Tjendrakasih, Suty Karno',
            'Shay Mitchell; Liza Soberano; Jon Jon Briones; Darren Criss; Manny Jacinto; Dante Basco',
        ],
    })

    output = polars.select(split_by_chars(input.get_column('input'), characters=',;')).to_series().to_list()

    expected = [
        'Nadya Arina', 'Refal Hady', 'Giorgino Abraham', 'Anggika Bolsterli', 'Laura Theux', 'Christine Hakim',
        'Maudy Koesnaedi', 'Rano Karno', 'Cornelia Agatha', 'Mandra Naih', 'Aminah Tjendrakasih', 'Suty Karno',
        'Shay Mitchell', 'Liza Soberano', 'Jon Jon Briones', 'Darren Criss', 'Manny Jacinto', 'Dante Basco',
    ]

    assert output == expected